import json

from django.core.mail import mail_admins
from django.http import HttpResponse, HttpResponseBadRequest
from django.template import loader, Context
from django.views.decorators.http import require_POST
from django.views.decorators.csrf import csrf_exempt

from csp import build_policy


@csrf_exempt
@require_POST
def report(request):
    """
    Accept a Content Security Policy violation report and forward
    the report via email to ADMINS.

    """

    try:
        violation = json.loads(request.raw_post_data)['csp-report']
    except Exception:
        return HttpResponseBadRequest()

    data = {}
    for key in violation:
        data[key.replace('-', '_')] = violation[key]

    c = Context(data)
    t = loader.get_template('csp/email/report.ltxt')
    body = t.render(c)

    mail_admins('CSP Violation', body)
    return HttpResponse()


def policy(request):
    """
    Returns a valid policy-uri, as an alternative to putting the whole
    policy in the header.

    """

    policy = build_policy()

    return HttpResponse(policy, mimetype='text/x-content-security-policy')



from django.conf import settings


def build_policy():
    """Builds the policy as a string from the settings."""

    policy = ['allow %s' % (' '.join(settings.CSP_ALLOW) if
                            hasattr(settings, 'CSP_ALLOW') else
                            "'self'")]
    if hasattr(settings, 'CSP_OPTIONS'):
        policy.append('options %s' % ' '.join(settings.CSP_OPTIONS))
    if hasattr(settings, 'CSP_IMG_SRC'):
        policy.append('img-src %s' % ' '.join(settings.CSP_IMG_SRC))
    if hasattr(settings, 'CSP_SCRIPT_SRC'):
        policy.append('script-src %s' % ' '.join(settings.CSP_SCRIPT_SRC))
    if hasattr(settings, 'CSP_OBJECT_SRC'):
        policy.append('object-src %s' % ' '.join(settings.CSP_OBJECT_SRC))
    if hasattr(settings, 'CSP_MEDIA_SRC'):
        policy.append('media-src %s' % ' '.join(settings.CSP_MEDIA_SRC))
    if hasattr(settings, 'CSP_FRAME_SRC'):
        policy.append('frame-src %s' % ' '.join(settings.CSP_FRAME_SRC))
    if hasattr(settings, 'CSP_FONT_SRC'):
        policy.append('font-src %s' % ' '.join(settings.CSP_FONT_SRC))
    if hasattr(settings, 'CSP_XHR_SRC'):
        policy.append('xhr-src %s' % ' '.join(settings.CSP_XHR_SRC))
    if hasattr(settings, 'CSP_STYLE_SRC'):
        policy.append('style-src %s' % ' '.join(settings.CSP_STYLE_SRC))
    if hasattr(settings, 'CSP_FRAME_ANCESTORS'):
        policy.append('frame-ancestors %s' %
                      ' '.join(settings.CSP_FRAME_ANCESTORS))
    if hasattr(settings, 'CSP_REPORT_URI'):
        policy.append('report-uri %s' % settings.CSP_REPORT_URI)

    return '; '.join(policy)



from setuptools import setup

setup(
    name='django_csp',
    version='1.0',
    description='Django Content Security Policy support.',
    author='James Socol',
    author_email='james@mozilla.com',
    url='http://github.com/mozilla/django-csp',
    license='BSD',
    packages=['csp'],
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Environment :: Web Environment :: Mozilla',
        'Intended Audience :: Developers',
        'Framework :: Django',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



from django.conf import settings

from csp import build_policy


class CSPMiddleware(object):
    """
    Implements the X-Content-Security-Policy response header, which
    conforming user-agents can use to restrict the permitted sources
    of various content.

    See https://wiki.mozilla.org/Security/CSP/Specification

    """

    def process_response(self, request, response):
        header = 'X-Content-Security-Policy'
        if getattr(settings, 'CSP_REPORT_ONLY', False):
            header = 'X-Content-Security-Policy-Report-Only'

        if header in response:
            # Don't overwrite existing headers.
            return response

        if getattr(settings, 'CSP_POLICY_URI', False):
            policy = 'policy-uri ' + settings.CSP_POLICY_URI
        else:
            policy = build_policy()
        response['X-Content-Security-Policy'] = policy
        return response
